@@ -931,7 +931,6 @@ TEST_LIST = [
         'success':          SUCCESS_DEFAULT,
         'pre_job':          None,
         'pre_success':      None,
-        'pre_success':      SUCCESS_DEFAULT,
         'requirements':     [Requirements.linux, Requirements.libaio],
     },
     {
@@ -941,7 +940,6 @@ TEST_LIST = [
         'success':          SUCCESS_DEFAULT,
         'pre_job':          None,
         'pre_success':      None,
-        'pre_success':      SUCCESS_DEFAULT,
         'requirements':     [Requirements.linux, Requirements.libaio],
     },
     {
@@ -951,9 +949,26 @@ TEST_LIST = [
         'success':          SUCCESS_DEFAULT,
         'pre_job':          None,
         'pre_success':      None,
+        'requirements':     [],
+    },
+    {
+        'test_id':          36,
+        'test_class':       FioJobFileTest,
+        'job':              't0036-post.fio',
+        'success':          SUCCESS_DEFAULT,
+        'pre_job':          't0036-pre.fio',
         'pre_success':      SUCCESS_DEFAULT,
         'requirements':     [],
     },
+    {
+        'test_id':          37,
+        'test_class':       FioJobFileTest,
+        'job':              't0037-post.fio',
+        'success':          SUCCESS_DEFAULT,
+        'pre_job':          't0037-pre.fio',
+        'pre_success':      SUCCESS_DEFAULT,
+        'requirements':     [Requirements.linux, Requirements.libaio],
+    },
     {
         'test_id':          1000,
         'test_class':       FioExeTest,
@@ -1084,6 +1099,22 @@ TEST_LIST = [
         'success':          SUCCESS_DEFAULT,
         'requirements':     [Requirements.linux, Requirements.nvmecdev],
     },
+    {
+        'test_id':          1016,
+        'test_class':       FioExeTest,
+        'exe':              't/client_server.py',
+        'parameters':       ['-f', '{fio_path}'],
+        'success':          SUCCESS_DEFAULT,
+        'requirements':     [Requirements.linux],
+    },
+    {
+        'test_id':          1017,
+        'test_class':       FioExeTest,
+        'exe':              't/verify.py',
+        'parameters':       ['-f', '{fio_path}'],
+        'success':          SUCCESS_LONG,
+        'requirements':     [],
+    },
 ]
 
 
